package com.yixin.ms.server;

import cn.hutool.core.util.IdUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.AES;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.crypto.util.SzhzSm2Util;
import com.yixin.ms.server.dto.DeviceScanCodeDTO;
import com.yixin.ms.server.dto.DeviceScanIdCardDTO;
import com.yixin.ms.server.dto.PlaceRecordDTO;
import com.yixin.ms.server.dto.TokenDataDTO;
import com.yixin.ms.server.vo.DeviceScanIdCardVO;
import com.yixin.ms.server.vo.HealthResultVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

/**
 * @author: lileilei
 * @date: 2022/6/23 10:47
 * @description: 杭州“四码一屏”统一接口
 */
@Component
@Slf4j
public class PlaceHealthCodeServer {
    private static final String TOKEN_KEY = "health:code:token";
    @Value("${health.code.clientId}")
    private String clientId;

    @Value("${health.code.clientSecret}")
    private String clientSecret;

    @Value("${health.code.publicKey}")
    private String publicKey;

    @Value("${health.code.getTokenUrl}")
    private String getTokenUrl;

    @Value("${health.code.deviceScanIdCardUrl}")
    private String deviceScanIdCardUrl;

    @Value("${health.code.deviceScanIdCardEncryptUrl}")
    private String deviceScanIdCardEncryptUrl;

    @Value("${health.code.deviceScanCodeUrl}")
    private String deviceScanCodeUrl;

    @Value("${health.code.queryPlaceInfoBySnUrl}")
    private String queryPlaceInfoBySnUrl;

    @Value("${health.code.savePlaceRecordUrl}")
    private String savePlaceRecordUrl;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private RedisUtil redisUtil;

    /**
     * 获取token
     *
     * @return
     */
    public HealthResultVO<TokenDataDTO> getToken() {
        HealthResultVO healthResultVO = restTemplate.getForObject(getTokenUrl, HealthResultVO.class);
        log.info("获取token响应内容：{}", JSON.toJSONString(healthResultVO));
        healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), TokenDataDTO.class));
        return healthResultVO;
    }

    /**
     * 获取token
     *
     * @return
     */
    public HealthResultVO<TokenDataDTO> getToken(Boolean cleanSession) {
        HealthResultVO healthResultVO = restTemplate.getForObject(new StringBuilder().append(getTokenUrl).append("?cleanSession=").append(cleanSession).toString(), HealthResultVO.class);
        log.info("获取token响应内容：{}", JSON.toJSONString(healthResultVO));
        healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), TokenDataDTO.class));
        return healthResultVO;
    }


    /**
     * 扫人员证件查询场所码信息
     *
     * @return
     */
    public HealthResultVO<DeviceScanIdCardVO> deviceScanIdCard(DeviceScanIdCardDTO dto) {
        HealthResultVO healthResultVO = null;
        try {
            //认证token
            String token = this.getToken().getData().getToken();
            dto.setToken(token);
            dto.setBizNo(IdUtil.fastUUID());
            String secret32Str = SecureUtil.md5(clientSecret).toLowerCase();
            AES aes = SecureUtil.aes(secret32Str.getBytes());
            String idNum = aes.encryptBase64(dto.getIdNumber().getBytes());
            dto.setIdNumber(idNum);
            healthResultVO = restTemplate.postForObject(deviceScanIdCardUrl, dto, HealthResultVO.class);
            log.info("扫人员证件查询场所码信息返回内容: {}", JSON.toJSONString(healthResultVO));
            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), DeviceScanIdCardVO.class));
        } catch (Exception e) {
            log.error("扫人员证件查询场所码信息异常：{}", e.getMessage());
            e.printStackTrace();
            return healthResultVO;
        }
        return healthResultVO;
    }



    /**
     * 扫人员证件查询场所码信息(请求体加密)
     *
     * @return
     */
    public HealthResultVO<DeviceScanIdCardVO> deviceScanIdCardEncrypt(DeviceScanIdCardDTO dto) {
        HealthResultVO healthResultVO = null;
        try {
            //认证token
            String token = this.getToken().getData().getToken();
            dto.setToken(token);
            dto.setBizNo(IdUtil.fastUUID());
            dto.setIdNumber(dto.getIdNumber());
            String jsonReq = JSON.toJSONString(dto);
            //得到加密的jsonBody,公钥在文档中有，或者找数杭接口人获取
            String requestBody = SzhzSm2Util.encrypt(publicKey, jsonReq);
            //获取最终的传参的JSON串
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("scanIdCardBody", requestBody);
            healthResultVO = restTemplate.postForObject(deviceScanIdCardEncryptUrl, jsonObject, HealthResultVO.class);
            log.info("扫人员证件查询场所码信息返回内容: {}", JSON.toJSONString(healthResultVO));
            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), DeviceScanIdCardVO.class));
        } catch (Exception e) {
            log.error("扫人员证件查询场所码信息异常：{}", e.getMessage());
            e.printStackTrace();
            return healthResultVO;
        }
        return healthResultVO;
    }

    /**
     * 健康码查询场所码信息
     *
     * @return
     */
    public HealthResultVO<DeviceScanIdCardVO> deviceScanCode(DeviceScanCodeDTO dto) {
        HealthResultVO healthResultVO = null;
        try {
            //认证token
            String token = this.getToken().getData().getToken();
            dto.setToken(token);
            dto.setBizNo(IdUtil.fastUUID());
            healthResultVO = restTemplate.postForObject(deviceScanCodeUrl, dto, HealthResultVO.class);
            log.info("健康码查询场所码信息返回内容: {}", JSON.toJSONString(healthResultVO));
            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), DeviceScanIdCardVO.class));
        } catch (Exception e) {
            log.error("健康码查询场所码信息：{}", e.getMessage());
            e.printStackTrace();
            return healthResultVO;
        }
        return healthResultVO;
    }

    /**
     * 通过设备sn查询场所码信息
     */
    public RestResult<PlaceInfoVO> queryPlaceInfoBySn(String sn) {
        RestResult result = null;

        try {
            result = restTemplate.getForObject(new StringBuilder().append(queryPlaceInfoBySnUrl).append("?sn=").append(sn).toString(), RestResult.class);
            result.setData(JSON.parseObject(JSON.toJSONString(result.getData()), PlaceInfoVO.class));
            log.info("通过设备sn: {}, 查询场所码信息返回内容:{}", sn, JSON.toJSONString(result));
        } catch (Exception e) {
            log.error("通过设备sn查询场所码信息异常：{}", e.getMessage());
            e.printStackTrace();
            return result;
        }

        return result;
    }

    /**
     * 保存场所记录
     */
    public RestResult<PlaceInfoVO> savePlaceRecord(PlaceRecordDTO dto) {
        return restTemplate.postForObject(savePlaceRecordUrl, dto, RestResult.class);
    }
}
